unit ListWindow;

//---------------------------------------------------------------------------//
//    Copyright (c) 2008  Meindert Kuipers, Netherlands                      //
//    meindert@kuiprs.nl                  www.kuiprs.nl                      //
//                                                                           //
// This program is free software; you can redistribute it and/or             //
// modify it under the terms of the GNU General Public License               //
// as published by the Free Software Foundation; either version 2            //
// of the License, or (at your option) any later version.                    //
//                                                                           //
// This program is distributed in the hope that it will be useful,           //
// but WITHOUT ANY WARRANTY; without even the implied warranty of            //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             //
// GNU General Public License for more details.                              //
//                                                                           //
// You should have received a copy of the GNU General Public License         //
// along with this program; if not, write to the Free Software               //
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA//
//---------------------------------------------------------------------------//
//
//---------------------------------------------------------------------------//
//  LISTWINDOW.PAS                                                           //
//  Unit for displaying text output in a seperate window                     //
//  Ver  Date     Description                                                //
//  1.00 Apr 2007 First Version                                              //
//  1.01 Apr 2008 Added saving of window position                            //
//                Replaced RichEdit by Memo                                  //
//  1.20 Apr 2008 Added CTRL-A function                                      //
//  1.50 May 2008 Final release                                              //
//  1.51 Sep 2008 Changed saving of screen position                                                                         //
//---------------------------------------------------------------------------//

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, IniFiles;

type
  TFrmLister = class(TForm)
    EdtList: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure EdtListKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FrmLister: TFrmLister;

implementation

{$R *.dfm}


procedure TFrmLister.FormCreate(Sender: TObject);
 var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create( ChangeFileExt( Application.ExeName, '.INI' ) );
  try
    FrmLister.Top:= Ini.ReadInteger( 'WindowsPositions', 'ListerTop', -1);
    FrmLister.Left:= Ini.ReadInteger( 'WindowsPositions', 'ListerLeft', -1);
    FrmLister.Width := Ini.ReadInteger( 'WindowsPositions', 'ListerWidth', 500);
    FrmLister.Height := Ini.ReadInteger( 'WindowsPositions', 'ListerHeight', 600);
  finally
    Ini.Free
  end;
end;


procedure TFrmLister.FormDestroy(Sender: TObject);
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(ChangeFileExt(Application.ExeName, '.INI' ));
  try
    Ini.WriteInteger( 'WindowsPositions', 'ListerTop', FrmLister.Top);
    Ini.WriteInteger( 'WindowsPositions', 'ListerLeft', FrmLister.Left);
    Ini.WriteInteger( 'WindowsPositions', 'ListerWidth', FrmLister.Width);
    Ini.WriteInteger( 'WindowsPositions', 'ListerHeight', FrmLister.Height);
  finally
    Ini.Free;
  end;
end;


procedure TFrmLister.EdtListKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if ((ssCtrl in Shift) AND (Key = ord('A'))) then begin
    // CONTROL-A pressed
    EdtList.SelectAll;
    Key := 0;
  end;
end;

end.
