unit MLDL2000;

//---------------------------------------------------------------------------//
//    Copyright (c) 2005  Meindert Kuipers, Netherlands                      //
//    meindert@kuipers.to                www.kuipers.to                      //
//                                                                           //
// This program is free software; you can redistribute it and/or             //
// modify it under the terms of the GNU General Public License               //
// as published by the Free Software Foundation; either version 2            //
// of the License, or (at your option) any later version.                    //
//                                                                           //
// This program is distributed in the hope that it will be useful,           //
// but WITHOUT ANY WARRANTY; without even the implied warranty of            //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             //
// GNU General Public License for more details.                              //
//                                                                           //
// You should have received a copy of the GNU General Public License         //
// along with this program; if not, write to the Free Software               //
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA//
//---------------------------------------------------------------------------//
//
//---------------------------------------------------------------------------//
//  MLDL2000.PAS                                                             //
//  Main program for M2kM                                                    //
//  Ver  Date     Description                                                //
//  1.00 Nov 2005 First version for release                                  //
//  1.01 Nov 2006 Compiled for Turbo Explorer                                //
//                CPLD is powered down when leaving M2kM                     //
//                Removed ShareMem reference, so BORLNDMM.DDL is not needed  //
//  1.02 Oct 2007 BORLNDMM.DLL is still needed                               //
//                Removed unused items from menu                             //
//---------------------------------------------------------------------------//

interface

uses
//  ShareMem,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  D2XXUnit, ComCtrls, Menus, About, Globals, GlobalConstants, ExtCtrls, Procs, Procs_MLDL,
  StdCtrls, Buttons, Preferences;

type
  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    PnlStatus: TStatusBar;
    F1: TMenuItem;
    MenuExit: TMenuItem;
    Help1: TMenuItem;
    Tools1: TMenuItem;
    MenuBitBang: TMenuItem;
    MenuAbout: TMenuItem;
    Conne1: TMenuItem;
    Connect1: TMenuItem;
    Disconnect1: TMenuItem;
    MemoryEditor1: TMenuItem;
    CPLDUpgrade1: TMenuItem;
    MLDLTest1: TMenuItem;
    N1: TMenuItem;
    ROMSRHandler1: TMenuItem;
    N3: TMenuItem;
    Prefs: TMenuItem;
    N2: TMenuItem;
    procedure MenuAboutClick(Sender: TObject);
    procedure MenuBitBangClick(Sender: TObject);
    procedure MenuExitClick(Sender: TObject);
    procedure MainFormCreate(Sender: TObject);
    procedure MemoryEditor1Click(Sender: TObject);
    procedure ROMSRHandler1Click(Sender: TObject);
    procedure Preferences1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure InitMLDL(Sender: TObject);
    procedure CloseMLDL(Sender: TObject);
    procedure Details1Click(Sender: TObject);
    procedure F1Click(Sender: TObject);
    procedure Edit1Click(Sender: TObject);
    procedure Create1Click(Sender: TObject);
    procedure Restore1Click(Sender: TObject);
    procedure MLDLTest1Click(Sender: TObject);
    procedure RunScript1Click(Sender: TObject);
    procedure CPLDUpgrade1Click(Sender: TObject);
    procedure ErrorStatus1Click(Sender: TObject);
    procedure PrefsClick(Sender: TObject);

  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;


implementation

uses BitBang_Unit, MemEdit, ROMHandler, Jtag, Tester;

{$R *.DFM}


procedure TMainForm.MainFormCreate(Sender: TObject);
// Called on initialization
// Initialize Global Variables here
begin
  I_Error := 0;                    // Initialize Error variable
  PnlStatus.SimpleText := 'MLDL2000 not connected';
  MLDL_Enable_Error_Report := true; // Enable Error Windows
  FT_Enable_Error_Report := true;
  MLDL_Is_Connected := false;

  // Disable appropriate menu items

end;


procedure TMainForm.MenuAboutClick(Sender: TObject);
begin
  AboutBox.Caption := 'About' + Application.Title;
  AboutBox.ShowModal;
end;

procedure TMainForm.MenuBitBangClick(Sender: TObject);
begin
  BitBang.Show;
end;

procedure TMainForm.MenuExitClick(Sender: TObject);
begin
  Application.Terminate;
end;

procedure TMainForm.InitMLDL(Sender: TObject);
// Opens the MLDL2000 Device
begin
  if OpenMLDLPort then begin
    // MLDL2000 now opened
    MainForm.PnlStatus.SimpleText := MLDL_Description + '  S/N: ' + MLDL_Serial;
    MainForm.Disconnect1.Enabled := true;
    MainForm.Connect1.Enabled := false;
    MLDL_Is_Connected := true;
    InitMLDLComm;
  end else begin
    // Device not opened for whatever reason
    MainForm.PnlStatus.SimpleText := 'MLDL2000 not connected';
    MainForm.Disconnect1.Enabled := false;
    MainForm.Connect1.Enabled := true;
    MLDL_Is_Connected := false;
  end;
  MainForm.Update;
end;


procedure TMainForm.CloseMLDL(Sender: TObject);
// Opens the MLDL2000 Device
begin
  CloseMLDLPort;
  MainForm.PnlStatus.SimpleText := 'MLDL2000 not connected';
  MainForm.Disconnect1.Enabled := false;
  MainForm.Connect1.Enabled := true;
  MLDL_Is_Connected := false;
end;


procedure TMainForm.MemoryEditor1Click(Sender: TObject);
begin
  FRmMemEdit.Show;
end;


procedure TMainForm.ROMSRHandler1Click(Sender: TObject);
begin
  FrmROMHandler.Show;
end;

procedure TMainForm.PrefsClick(Sender: TObject);
begin
  FrmPreferences.Show;
end;

procedure TMainForm.Preferences1Click(Sender: TObject);
begin
  ShowMessage('This function is not implemented');
end;

procedure TMainForm.FormShow(Sender: TObject);
// Called when Mainform shows, automatically opens the MLDL2000
begin
  InitMLDL(Sender);
end;

procedure TMainForm.Details1Click(Sender: TObject);
begin
  ShowMessage('This function is not implemented');
end;

procedure TMainForm.F1Click(Sender: TObject);
begin
  ShowMessage('This function is not implemented');
end;

procedure TMainForm.Edit1Click(Sender: TObject);
begin
  ShowMessage('This function is not implemented');
end;

procedure TMainForm.Create1Click(Sender: TObject);
begin
  ShowMessage('This function is not implemented');
end;

procedure TMainForm.Restore1Click(Sender: TObject);
begin
  ShowMessage('This function is not implemented');
end;

procedure TMainForm.MLDLTest1Click(Sender: TObject);
begin
  // ShowMessage('This function is not implemented');
  Frm_Tester.Show;
end;

procedure TMainForm.RunScript1Click(Sender: TObject);
begin
  ShowMessage('This function is not implemented');
end;

procedure TMainForm.CPLDUpgrade1Click(Sender: TObject);
begin
  Frm_JTAG.Show;
end;

procedure TMainForm.ErrorStatus1Click(Sender: TObject);
begin
  ShowMessage('This function is not implemented');
end;

end.
