unit Procs;

//---------------------------------------------------------------------------//
//    Copyright (c) 2008  Meindert Kuipers, Netherlands                      //
//    meindert@kuiprs.nl                  www.kuiprs.nl                      //
//                                                                           //
// This program is free software; you can redistribute it and/or             //
// modify it under the terms of the GNU General Public License               //
// as published by the Free Software Foundation; either version 2            //
// of the License, or (at your option) any later version.                    //
//                                                                           //
// This program is distributed in the hope that it will be useful,           //
// but WITHOUT ANY WARRANTY; without even the implied warranty of            //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             //
// GNU General Public License for more details.                              //
//                                                                           //
// You should have received a copy of the GNU General Public License         //
// along with this program; if not, write to the Free Software               //
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA//
//---------------------------------------------------------------------------//
//
//---------------------------------------------------------------------------//
//  PROCS.PAS                                                                //
//  Various functions and procedures used in M2kM                            //
//  Ver  Date     Description                                                //
//  1.00 Nov 2005 First version for release                                  //
//  1.01 Nov 2006 Compiled for Turbo Explorer                                //
//  1.02 Apr 2007 Added function for HP User Characters                      //
//  1.20 Apr 2008 Added functions to assist disassembler comments            //
//                Added FileCopy                                             //
//  1.50 May 2008 Final release                                              //
//---------------------------------------------------------------------------//

interface

uses
  Globals, SysUtils, StrUtils, Classes ;

function StrStr(Len: Integer; ch: Char): AnyString;
// Returns a string wil 'Len' times the character 'ch'

function Hex1(Nyb: longword): Char;
// Returns one Hex character representing the value of 'Nyb'

function Hex2(Hex: word): String2;
// Returns 2 Hex characters representing the value of 'Hex'

function Hex3(Hex: word): String3;
// Returns 3 Hex characters representing the value of 'Hex'

function Hex4(Hex: word): String4;
// Returns 4 Hex characters representing the value of 'Hex'

function Hex5(Hex: LongWord): String;
// Returns 5 Hex characters representing the value of 'Hex'

function HexN(Hex: Longword; N: integer): String;
// Returns N Hex characters representing the value of 'Hex'

function SStr(I: Integer): String;
// wrapper for the Str procedure

function VVal(S: string): integer;
// wrapper for the Val procedure

function Int3(I: Integer): String3;
// Returns a string with the value of I with 3 digits

function Int2(I: Integer): String2;
// Returns a string with the value of I with 2 digits

function Bin4(Hex: word): String4;
// Returns a binary string with the value of Hex with 4 bits

function Bin8(Hex: word): String8;
// Returns a binary string with the value of Hex with 8 bits

function Bin16(Hex: word): String16;
// Returns a binary string with the value of Hex with 16 bits and a '.' as divider

function Bin10(Hex: word): String10;
// Returns a binary string with the value of Hex with 10 bits

function Asc(Hex: Byte): Char;
// Returns a printable Character for the value Hex and a '.' if non-printable

function HPChar(W: word): Char;
// Returns a HP Character for the value Hex

function HPUserChar(W: word): Char;
// Returns a HP Character for the Value HEX for UserCode

function HPCharList(W: word): Char;
// Returns a HP Character for the value Hex only for alphas and nums

function HexToInt(var Line: string): Integer;
// Converts a string, representing a hex number to its integer value
// HexToInt will find the first valid hex digit and continue until the
// first non-valid hex digit. Hex digits may be in upper- or lowercase.
// returns -1 in case of an error
// leading spaces and control characters are ignored
// the hex digits will be removed from the string

function HexToByte(var Line: string; var Rslt: boolean): Byte;
// converts the first valid hex characters in a string to a byte
// removes everything inluding the two byte from the string
// If there is only one hex digit this wil be converted

function BinToInt(var Line: string): Integer;
// Converts a string, representing a binary number to its integer value
// HexToInt will find the first valid binary digit and continue until the
// first non-valid binary digit.
// returns -1 in case of an error
// leading spaces and control characters are ignored
// the binary digits will be removed from the string

function IntToHex(I: integer): String;
// Converts a number to its hexadecimal representation
// I is the number to be converted, N the number of characters

function GetFirstWord(var Line: string): string;
// Returnes the first complete word

procedure ExpandSpaces(var Line: string; N: integer);
// expands the string Line with spaces until the length is N
// will do nothing if string is already longer or when N >255

procedure FileCopy(const FSrc, FDst: string);
// Copies a file from FSrc to FDst


implementation


function StrStr(Len: Integer; ch: Char): AnyString;
begin
  StrStr := StringOfChar(ch, Len);
end; { StrStr }


function Hex1(Nyb: longword): Char;
const
  N: string[16] = '0123456789ABCDEF';
begin
  Hex1 := N[(Nyb and $000F) + 1];
end; { Nybble }


function Hex2(Hex: word): String2;
var
  S: String2;
begin
  S[0] := #02;
  S[1] := Hex1(Hex shr 4);
  S[2] := Hex1(Hex);
  Hex2 := S;
end; { Hex2 }


function Hex3(Hex: word): String3;
var
  S: String3;
begin
  S[0] := #03;
  S[1] := Hex1(Hex shr 8);
  S[2] := Hex1(Hex shr 4);
  S[3] := Hex1(Hex);
  Hex3 := S;
end; { Hex3 }


function Hex4(Hex: word): String4;
var
  S: String4;
begin
  S[0] := #04;
  S[1] := Hex1(Hex shr 12);
  S[2] := Hex1(Hex shr 8);
  S[3] := Hex1(Hex shr 4);
  S[4] := Hex1(Hex);
  Hex4 := S;
end; { Hex4 }

function Hex5(Hex: LongWord): String;
var
  S: String;
begin
  SetLength(S, 5);
  S[1] := Hex1(Hex shr 16);
  S[2] := Hex1(Hex shr 12);
  S[3] := Hex1(Hex shr 8);
  S[4] := Hex1(Hex shr 4);
  S[5] := Hex1(Hex);
  Hex5 := S;
end;

function HexN(Hex: Longword; N: integer): String;
var
  S: String;
  i: integer;
begin
  S := '';
  for i := (N - 1) downto 0 do S := S + Hex1(Hex shr (i * 4));
  HexN := S;
end;

function SStr(I: Integer): String;
// wrapper for the Str procedure
var
  Hulp: String;
begin
  Str(I, Hulp);
  SStr := Hulp;
end;

function VVal(S: string): integer;
// wrapper for the Val procedure
var
  I, Code: integer;
begin
  Val(S, I, Code);
  VVal := I;
end;


function Int2(I: Integer): String2;
var
  Hulp: String2;
begin
  Str(I: 2, Hulp);
  if Hulp[1] = ' ' then Hulp[1] := '0';
  Int2 := Hulp;
end;

function Int3(I: Integer): String3;
var
  Hulp: String3;
begin
  if I > 999 then I := 999;
  Str(I: 3, Hulp);
  if Hulp[1] = ' ' then Hulp[1] := '0';
  if Hulp[2] = ' ' then Hulp[2] := '0';
  Int3 := Hulp;
end;

function Bin4(Hex: word): String4;
// Returns a binary string with the value of I with 4 bits
var
  Hulp: String4;
begin
  Hex := Hex and $000F;
  case Hex of
    $0000 : Hulp := '0000';
    $0001 : Hulp := '0001';
    $0002 : Hulp := '0010';
    $0003 : Hulp := '0011';
    $0004 : Hulp := '0100';
    $0005 : Hulp := '0101';
    $0006 : Hulp := '0110';
    $0007 : Hulp := '0111';
    $0008 : Hulp := '1000';
    $0009 : Hulp := '1001';
    $000A : Hulp := '1010';
    $000B : Hulp := '1011';
    $000C : Hulp := '1100';
    $000D : Hulp := '1101';
    $000E : Hulp := '1110';
    $000F : Hulp := '1111';
  end;
  Bin4 := Hulp;
end;


function Bin8(Hex: word): String8;
// Returns a binary string with the value of I with 8 bits
begin
  Bin8 := Bin4(Hex shr 4) + Bin4(Hex);
end;


function Bin16(Hex: word): String16;
// Returns a binary string with the value of I with 16 bits and a '.' as divider
begin
  Bin16 := Bin8(Hex shr 8) + '.' + Bin8(Hex);
end;


function Bin10(Hex: word): String10;
// Returns a binary string with the value of I with 10 bits
var S : string16;
begin
  S := Bin8(Hex shr 8) + Bin8(Hex);
  Bin10 := RightStr(S, 10);
end;


function Asc(Hex: Byte): Char;
begin
  if ((Hex > 31) and (Hex < 127)) then Asc := Chr(Hex)
  else Asc := '.';
end; { Asc }


function HPChar(W: word): Char;
const
  CharS: string[64] =
    '@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_ |"#$%&`<>*+{-}/0123456789,<=>?';
begin
  HPChar := CharS[(W and $003F) + 1];
end;


function HPUserChar(W: word): Char;
const
  CharS: array[0..127] of Integer = (
    004, 120, 120, 027, 224, 225, 226, 025, 127, 229, 004, 076, 230, 023, 231, 232,
    233, 234, 235, 143, 134, 142, 132, 153, 148, 154, 129, 146, 145, 216, 156, 178,
    032, 033, 034, 035, 036, 037, 038, 039, 040, 041, 042, 043, 044, 045, 046, 047,
    048, 049, 050, 051, 052, 053, 054, 055, 056, 057, 058, 059, 060, 061, 062, 063,
    064, 065, 066, 067, 068, 069, 070, 071, 072, 073, 074, 075, 076, 077, 078, 079,
    080, 081, 082, 083, 084, 085, 086, 087, 088, 089, 090, 091, 092, 093, 094, 095,
    096, 097, 098, 099, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111,
    112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 227, 124, 026, 228, 195);
begin
  HPUserChar := Char(CharS[(W and $007F)]);
end;


function HPCharList(W: word): Char;
const
  CharS: string[64] =
    '.ABCDEFGHIJKLMNOPQRSTUVWXYZ.....................0123456789......';
begin
  HPCharList := CharS[(W and $003F) + 1];
end;


function HexToInt(var Line: string): integer;
  { Converts a string, representing a hex number to its integer value
    HexToInt will find the first valid hex digit and continue until the
    first non-valid hex digit. Hex digits may be in upper- or lowercase.
    returns -1 in case of an error
    leading spaces and control characters are ignored
    the hex digits will be removed from the string }
var
  L, Res, i: integer;
  D: Char;
  ErrFlag, EndFlag: boolean;
begin
  Res := -1;
  ErrFlag := False;
  EndFlag := False;
  L := Length(Line);
  i := 0;
  if L > 0 then begin
    Line := TrimLeft(Line); { remove leading spaces and control characters }
    i := 1;
    while (i <= L) and not EndFlag do begin
      D := UpCase(Line[i]);
      if D in HexSet then begin
        if Res < 0 then Res := 0;
        if D in DecSet then begin
          Res := Res * 16 + Ord(D) - Ord('0');
        end else if D in HexChr then begin
          Res := Res * 16 + Ord(D) - Ord('A') + 10;
        end;
      end else if Res >= 0 then EndFlag := true;
      i := i + 1;
    end;
  end else ErrFlag := True; // in case of empty string or no hex characters found
  if ErrFlag then Res := -1 else Delete(Line, 1, i - 1);
  HexToInt := Res;
end;

function GetFirstWord(var Line: string): string;
// Reads the first complete word from a string, removes the word from the original string
// if the word is surrounded by quotes, will go on to next quote to include spaces
// quotes can be "double" or 'single' quotes
// if quotes are used, a quote must be the first character
// only spaces and tabs count as separator, all other ascii chars count!
// if string was empty, or no valid word found, an empty string will be returned
var
  L, i: integer;
  S: string;
  EndFlag, UseQuote: boolean;
  D, Q: Char;
begin
  S := '';                    // Our result string
  Line := TrimLeft(Line);  // remove all leading spaces and control characters
  L := Length(Line);
  UseQuote := false;
  EndFlag := false;
  Q := Char(0);
  if L > 0 then begin
    i := 1;
    while (i <= L) and (not EndFlag) do begin
      D := Line[i];
      if D in QuoteSet then begin
        // string is quoted, so be careful
        if UseQuote and (D = Q) then begin
          EndFlag := true; // this is our closing quote
          inc(i);
        end else begin
          // first quote in string
          UseQuote := true;
          Q := D;         // to remember quote character
          inc(i);
        end;
      end else begin
        // not a quote character
        if (D = SPACE) or (D = TAB) then
          EndFlag := true
        else begin
          // no space or tab, so this is real!
          S := S + D;
          inc(i);
        end;
      end;
    end;
    Delete(Line, 1, i - 1);  // remove string (and quotes) from the original string
    Line := TrimLeft(Line);  // remove any other rubbish from our string
  end;
  GetFirstWord := S;
end;


function HexToByte(var Line: string; var Rslt: boolean): Byte;
// converts the two first valid hex characters in a string to a byte
// removes everything including the two byte from the string
// If there is only one hex digit this wil be converted
var
  L, Res, i, FirstHex: integer;
  D: Char;
  ErrFlag, EndFlag: boolean;
begin
  Res := 0;
  D := Chr(0);
  Rslt := false;
  FirstHex := 0;
  ErrFlag := False;
  EndFlag := False;
  L := Length(Line);
  i := 0;
  if L > 0 then begin
    Line := TrimLeft(Line); { remove leading spaces and control characters }
    i := 1;
    while (i <= L) and not EndFlag do begin
      D := UpCase(Line[i]);
      if D in HexSet then begin  // first hex character found
        FirstHex := i;
        EndFlag := true;
      end else
        i := i + 1;
    end;
    // First Hex is now found or we have an error or end of line
    if FirstHex > 0 then begin
      // Hex digit found!
      if D in DecSet then
        Res := Ord(D) - Ord('0')
      else
        Res := Ord(D) - Ord('A') + 10;
      // process second charachter
      i := i + 1;
      if i <= L then
        D := UpCase(Line[i]);
        if D in HexSet then begin
          i := i + 1;
          if D in DecSet then
             Res := 16 * Res + Ord(D) - Ord('0')
          else
             Res := 16 * Res + Ord(D) - Ord('A') + 10;
        end;
    end else ErrFlag := True; // in case of empty string or no hex characters found
  end else ErrFlag := true;
  if ErrFlag then Rslt := false else begin
    Delete(Line, 1, i - 1);
    Rslt := true;
  end;
  HexToByte := Res;
end;


function BinToInt(var Line: string): Integer;
// Converts a string, representing a binary number to its integer value
// HexToInt will find the first valid binary digit and continue until the
// first non-valid binary digit.
// returns -1 in case of an error
// leading spaces and control characters are ignored
// the binary digits will be removed from the string
var
  L, Res, i: integer;
  D: Char;
  ErrFlag, EndFlag: boolean;
begin
  Res := -1;
  ErrFlag := False;
  EndFlag := False;
  L := Length(Line);
  i := 0;
  if L > 0 then begin
    Line := TrimLeft(Line); { remove leading spaces and control characters }
    i := 1;
    while (i <= L) and not EndFlag do begin
      D := Line[i];
      case D of
        '0': if Res < 0 then Res := 0 else Res := Res * 2;
        '1': Res := Res * 2 + 1;
        else if Res >= 0 then EndFlag := true;
      end;
      i := i + 1;
    end;
  end else ErrFlag := True; // in case of empty string or no bin characters found
  if ErrFlag then Res := -1 else Delete(Line, 1, i - 1);
  BinToInt := Res;
end;


function IntToHex(I: integer): String;
// Converts a number to its hexadecimal representation
// I must be positive
var
  S: String;
begin
  S := '';
  while I > 0 do begin
    Insert(Hex1(I mod 16), S, 1);
    I := I div 16;
  end;
  IntToHex := S;
end;


procedure ExpandSpaces(var Line: string; N: integer);
// expands the string Line with spaces until the length is N
// will do nothing if string is already longer or when N >255
var
  i, L: integer;
begin
  L := Length(Line);
  if (L > N) or (N > 255) then
    Exit
  else
    for i := L to N do Line := Line + ' ';
end;


procedure FileCopy(const FSrc, FDst: string);
// Copy a file FSrc to FDst
var
  sStream, dStream: TFileStream;
begin
  sStream := TFileStream.Create(FSrc, fmOpenRead);
  try
    dStream := TFileStream.Create(FDst, fmCreate);
    try
      dStream.CopyFrom(sStream, 0);
    finally
      dStream.Free;
    end;
  finally
    sStream.Free;
  end;
end;


end.

