unit Preferences;

//---------------------------------------------------------------------------//
//    Copyright (c) 2008  Meindert Kuipers, Netherlands                      //
//    meindert@kuiprs.nl                  www.kuiprs.nl                      //
//                                                                           //
// This program is free software; you can redistribute it and/or             //
// modify it under the terms of the GNU General Public License               //
// as published by the Free Software Foundation; either version 2            //
// of the License, or (at your option) any later version.                    //
//                                                                           //
// This program is distributed in the hope that it will be useful,           //
// but WITHOUT ANY WARRANTY; without even the implied warranty of            //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             //
// GNU General Public License for more details.                              //
//                                                                           //
// You should have received a copy of the GNU General Public License         //
// along with this program; if not, write to the Free Software               //
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA//
//---------------------------------------------------------------------------//
//
//---------------------------------------------------------------------------//
//  PREFERENCES.PAS                                                          //
//  Handling of Preferences and .INI file                                    //
//  Ver  Date     Description                                                //
//  0.10 Mar 2008 First Version                                              //
//  1.20 Apr 2008 Changes structure, added some variables                    //
//       Apr 2008 Added SkipNOPS                                             //
//  1.50 May 2008 Final release                                              //
//  1.51 Sep 2008 Changed saving of screen position                          //
//---------------------------------------------------------------------------//


interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Forms,
  Dialogs, StdCtrls, ExtCtrls, Globals, GlobalConstants, IniFiles, Procs,
  BitBang_Unit, About, MemEdit, Procs_MLDL, Jtag, Tester, Romdis,
  ListWindow, ModDialog, Controls;

type
  TFrmPreferences = class(TForm)
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    ChkComments: TCheckBox;
    ChkAutoLabel: TCheckBox;
    ChkMainFrame: TCheckBox;
    ChkXROM: TCheckBox;
    GrpMnem: TRadioGroup;
    EdtCommSpeed: TEdit;
    EdtJTAGSpeed: TEdit;
    EdtUSBTO: TEdit;
    EdtFLASHTO: TEdit;
    EdtSecTO: TEdit;
    EdtALLTO: TEdit;
    Label1: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label2: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    ChkAutoVerify: TCheckBox;
    ChkConfDown: TCheckBox;
    BtnOK: TButton;
    BtnCancel: TButton;
    BtnApply: TButton;
    LblMPSSE: TLabel;
    LblJTAG: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    CmbBlockSize: TComboBox;
    ChkAutoFind: TCheckBox;
    ChkSRAMDefault: TCheckBox;
    ChkSkipNOPS: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure BtnApplyClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure BtnOKClick(Sender: TObject);
    procedure BtnCancelClick(Sender: TObject);
    procedure EdtCommSpeedChange(Sender: TObject);
    procedure EdtJTAGSpeedChange(Sender: TObject);
    procedure SaveIni;
    procedure ReadIni;
    procedure FormDestroy(Sender: TObject);

  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FrmPreferences: TFrmPreferences;
  MPSSEValue, JTAGValue: integer;

implementation

uses ROMHandler;

{$R *.dfm}


procedure TFrmPreferences.SaveIni;
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(ChangeFileExt(Application.ExeName, '.INI' ));
  try
    // Disassembler section
    Ini.WriteBool('Disassembler', 'Comments', PrefComments);
                   // if true, will generate auto comments in
    Ini.WriteBool('Disassembler', 'AutoLabel', PrefAutoLabel);
                   // if true, will generate auto labels for jump addresses
    Ini.WriteBool('Disassembler', 'MainFrame', PrefMainFrame);
                   // if true, will use Mainframe label file
    Ini.WriteBool('Disassembler', 'XROM', PrefXROM);
                   // if true, will use XROM file
    Ini.WriteBool('Disassembler', 'SkipNOPS', PrefSkipNOPs);
                   // if true, skips NOPs
    Ini.WriteInteger('Disassembler', 'Mnemonics', PrefMnem);
                   // indicates Mnemonics type:  0 - Jacobs/De Arras, Default
                   //                            1 - HP
                   //                            2 - ZenCode
                   //                            3 - SDK41 (JDA only)

    // Communication Section
    Ini.WriteInteger('Communication', 'CommSpeed', PrefCommSpeed);
                      // MPSSE Communication Speed $0000 - $FFFF
    Ini.WriteInteger('Communication', 'JTAGSpeed', PrefJTAGSpeed);
                      // MPSSE Speed for use with JTAG: $0000 - $FFFF
    Ini.WriteInteger('Communication', 'USBTimeOut', PrefUSBTO);
                      // regular USB timeout in msec
    Ini.WriteInteger('Communication', 'FLASHTimeout', PrefFLASHTO);
                      // single byte FLASH write timeout in msec
    Ini.WriteInteger('Communication', 'SectorTimeout', PrefSecTO);
                      // FLASH Sector Erase timeout in msec
    Ini.WriteInteger('Communication', 'EraseAllTimeout', PrefALLTO);
                      // FLASH ALl Erase timeout in seconds !
    Ini.WriteInteger('Communication', 'Blocksize', PrefBlockSize);
                      // MPSSE Communication block size

    // User Interface Section
    // Enable Automatic Verify after read/write
    Ini.WriteBool('UserInterface', 'AutoVerify', PrefAutoVerify);
    // Enable Confirm on Download
    Ini.WriteBool('UserInterface', 'ConfirmDownload', PrefConfDown);
    Ini.WriteBool('UserInterface', 'AutoFindROM', PrefAutoFind);
    Ini.WriteBool('UserInterface', 'AutoFindROM', PrefMemType);

  finally
    Ini.Free;
  end;
end;


procedure TFrmPreferences.ReadIni;
 var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create( ChangeFileExt( Application.ExeName, '.INI' ) );
  try
    // Disassembler section
    PrefComments   := Ini.ReadBool('Disassembler', 'Comments', true);
                     // if true, will generate auto comments in
    PrefAutoLabel  := Ini.ReadBool('Disassembler', 'AutoLabel', true);
                     // if true, will generate auto labels for jump addresses
    PrefMainFrame  := Ini.ReadBool('Disassembler', 'MainFrame', false);
                     // if true, will use Mainframe label file
    PrefXROM       := Ini.ReadBool('Disassembler', 'XROM', true);
                     // if true, will use XROM file
    PrefSkipNOPs   := Ini.ReadBool('Disassembler', 'SkipNOPs', true);

    PrefMnem       := Ini.ReadInteger('Disassembler', 'Mnemonics', 0);
                   // indicates Mnemonics type:  0 - Jacobs/De Arras, Default
                   //                            1 - HP
                   //                            2 - ZenCode
                   //                            3 - SDK41 (JDA only)

    // Communication Section
    MPSSEValue  := Ini.ReadInteger('Communication', 'CommSpeed', MPSSE_Speed);
                     // MPSSE Communication Speed $0000 - $FFFF
    if (MPSSEValue >= 0) and (MPSSEValue < 65535) then
      PrefCommSpeed  := MPSSEValue
    else
      PrefCommSpeed  := MPSSE_Speed;

    JTAGValue        := Ini.ReadInteger('Communication', 'JTAGSpeed', 20);
                     // MPSSE Speed for use with JTAG: $0000 - $FFFF
    if (JTAGValue >= 0) and (JTAGValue < 65535) then
      PrefJTAGSpeed  := JTAGValue
    else
      PrefJTAGSpeed  := JTAGValue;

    PrefUSBTO      := Ini.ReadInteger('Communication', 'USBTimeOut', TimeOut_USB);
                     // regular USB timeout in msec
    PrefFLASHTO    := Ini.ReadInteger('Communication', 'FLASHTimeout', TimeOut_Flash_Byte);
                     // single byte FLASH write timeout in msec
    PrefSecTO      := Ini.ReadInteger('Communication', 'SectorTimeout', TimeOut_FLASH_Sector);
                     // FLASH Sector Erase timeout in msec
    PrefALLTO      := Ini.ReadInteger('Communication', 'EraseAllTimeout', TimeOut_FLASH_Erase);
                     // FLASH ALl Erase timeout in seconds !
    PrefBlockSize  := Ini.ReadInteger('Communication', 'Blocksize', 256);
                     // MPSSE Communication block size

    // User Interface Section
    PrefAutoVerify := Ini.ReadBool('UserInterface', 'AutoVerify', true);
    // Enable Automatic Verify after read/write
    PrefConfDown   := Ini.ReadBool('UserInterface', 'ConfirmDownload', true);
    // Enable Confirm on Download
    PrefAutoFind   := Ini.ReadBool('UserInterface', 'AutoFindROM', true);
    PrefMemType    := Ini.ReadBool('UserInterface', 'MemoryTypeSRAM', true);

  finally
    Ini.Free
  end;
end;


procedure TFrmPreferences.BtnApplyClick(Sender: TObject);
// Apply Changes and write these back to the INI file
var
  Code: integer;
begin
  PrefComments   := ChkComments.Checked;
  PrefAutoLabel  := ChkAutoLabel.Checked;
  PrefMainFrame  := ChkMainFrame.Checked;
  PrefXROM       := ChkXROM.Checked;
  PrefSkipNOPs   := ChkSkipNOPs.Checked;
  PrefMnem       := GrpMnem.ItemIndex;

  PrefAutoVerify := ChkAutoVerify.Checked;
  PrefConfDown   := ChkConfDown.Checked;
  PrefAutoFind   := ChkAutoFind.Checked;
  FrmROMHandler.ChkAuto.Checked := PrefAutoFind;
  PrefMemType    := ChkSRAMDefault.Checked;

  if PrefMemType then with FrmROMHandler do begin
    // Memory Type is SRAM
    RadioSRAM.Checked := true;
    RadioFLASH.Checked := false;
    MemTp := SRAM;
    PnlMemType.Caption := 'SRAM';
    Val(EdtROMNum.Text, I, Code);
    if I > 63 then EdtROMNum.Text := '63';
    UpDnNumber.Max := 63;
  end else with FrmROMHandler do begin
    // Memory Type is FLASH
    RadioSRAM.Checked := false;
    RadioFLASH.Checked := true;
    MemTp := FLASH;
    PnlMemType.Caption := 'FLASH';
    UpDnNumber.Max := 255;
  end;
  if MLDL_Is_Connected and FrmROMHandler.ChkAuto.Checked then
    FrmROMHandler.ChkAutoClick(Sender);

  if (MPSSEValue >= 0) and (MPSSEValue < 65535) then
    PrefCommSpeed  := MPSSEValue
  else
    PrefCommSpeed  := MPSSE_Speed;

  // Now change the MPSSE Speed
  if PortIsOpen then begin
    CloseMLDLPort;
    if OpenMLDLPort then InitMLDLComm;
  end;

  if (JTAGValue >= 0) and (JTAGValue < 65535) then
    PrefJTAGSpeed  := JTAGValue
  else
    PrefJTAGSpeed  := JTAGValue;

  Val(EdtUSBTO.Text, PrefUSBTO, Code);
  Val(EdtFLASHTO.Text, PrefFLASHTO, Code);
  Val(EdtSECTO.Text, PrefSecTO, Code);
  Val(EdtALLTO.Text, PrefALLTO, Code);

  case CmbBlockSize.ItemIndex of
    0:   PrefBlockSize :=   16;
    1:   PrefBlockSize :=   32;
    2:   PrefBlockSize :=   64;
    3:   PrefBlockSize :=  128;
    4:   PrefBlockSize :=  256;
    5:   PrefBlockSize :=  512;
    6:   PrefBlockSize := 1024;
    else PrefBlockSize :=  256;
  end;

  if PrefUSBTO     = 0 then PrefUSBTO     := TimeOut_USB;
  if PrefFLASHTO   = 0 then PrefFLASHTO   := TimeOut_Flash_Byte;
  if PrefSecTO     = 0 then PrefSecTO     := TimeOut_FLASH_Sector;
  if PrefALLTO     = 0 then PrefALLTO     := TimeOut_FLASH_Erase;
  if PrefBlockSize = 0 then PrefBlockSize := 256;

  SaveIni;

end;


procedure TFrmPreferences.BtnCancelClick(Sender: TObject);
// Close window, first read old values from INI file
begin
  ReadIni;
  FrmPreferences.Close;
end;

procedure TFrmPreferences.BtnOKClick(Sender: TObject);
begin
  FrmPreferences.BtnApplyClick(Sender);
  FrmPreferences.Close;
end;


procedure TFrmPreferences.EdtCommSpeedChange(Sender: TObject);
// Called when the Edit changes, show calculated bit rate
var
  bitrate: real;
  Code: integer;
  S, SS: string;
begin
  Val(EdtCommSpeed.Text, MPSSEValue, Code);
  if (Code = 0) and (MPSSEValue >= 0) and (MPSSEValue <= 65353) then begin
    // Value was OK
    bitrate := 12 / (2 * (1 + MPSSEValue));
    if bitrate > 1 then begin
      SS := 'mbit/s'
    end else begin
      bitrate := bitrate * 1000;
      SS := 'kbit/s';
    end;
    if bitrate < 1 then begin
      bitrate := bitrate * 1000;
      SS := 'bit/s';
    end;
    //    if bitrate >  then
    Str(bitrate: 5:2, S);
    LblMPSSE.Caption := S + ' ' + SS;
  end else begin
    // Code was not correct, remove offending character
    LblMPSSE.Caption := 'wrong value';
    MPSSEValue := -1;
  end;
end;


procedure TFrmPreferences.EdtJTAGSpeedChange(Sender: TObject);
var
  bitrate: real;
  Code: integer;
  S, SS: string;
begin
  Val(EdtJTAGSpeed.Text, JTAGValue, Code);
  if (Code = 0) and (JTAGValue >= 0) and (JTAGValue <= 65353) then begin
    // Value was OK
    bitrate := 12 / (2 * (1 + JTAGValue));
    if bitrate > 1 then begin
      SS := 'mbit/s'
    end else begin
      bitrate := bitrate * 1000;
      SS := 'kbit/s';
    end;
    if bitrate < 1 then begin
      bitrate := bitrate * 1000;
      SS := 'bit/s';
    end;
    //    if bitrate >  then
    Str(bitrate: 5:2, S);
    LblJTAG.Caption := S + ' ' + SS;
  end else begin
    // Code was not correct, remove offending character
    LblJTAG.Caption := 'wrong value';
    JTAGValue := -1;
  end;
end;


procedure TFrmPreferences.FormShow(Sender: TObject);
// Show current values in the Checkboxes
begin
  ReadIni;
  ChkComments.Checked   := PrefComments;
  ChkAutoLabel.Checked  := PrefAutoLabel;
  ChkMainFrame.Checked  := PrefMainFrame;
  ChkXROM.Checked       := PrefXROM;
  ChkSkipNOPs.Checked   := PrefSkipNOPs;
  GrpMnem.ItemIndex     := PrefMnem;

  EdtCommSpeed.Text     := SStr(PrefCommSpeed);
  EdtJTAGSpeed.Text     := SStr(PrefJTAGSPeed);

  ChkAutoVerify.Checked := PrefAutoVerify;
  ChkConfDown.Checked   := PrefConfDown;
  ChkAutoFind.Checked   := PrefAutoFind;

  case PrefBlockSize of
      16: CmbBlockSize.ItemIndex := 0;
      32: CmbBlockSize.ItemIndex := 1;
      64: CmbBlockSize.ItemIndex := 2;
     128: CmbBlockSize.ItemIndex := 3;
     256: CmbBlockSize.ItemIndex := 4;
     512: CmbBlockSize.ItemIndex := 5;
    1024: CmbBlockSize.ItemIndex := 6;
    else  CmbBlockSize.ItemIndex := 4;
  end;

  EdtUSBTO.Text   := SStr(PrefUSBTO);
  EdtFLASHTO.Text := SStr(PrefFLASHTO);
  EdtSECTO.Text   := SStr(PrefSecTO);
  EdtALLTO.Text   := SStr(PrefALLTO);

end;


procedure TFrmPreferences.FormCreate(Sender: TObject);
// Read Values from INI File into Global Variables
// Creates the file if it does not exist
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create( ChangeFileExt( Application.ExeName, '.INI' ) );
  try
    FrmPreferences.Top  := Ini.ReadInteger( 'WindowsPositions', 'PrefTop', -1);
    FrmPreferences.Left := Ini.ReadInteger( 'WindowsPositions', 'PrefLeft', -1);
  finally
    Ini.Free
  end;
end;


procedure TFrmPreferences.FormDestroy(Sender: TObject);
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(ChangeFileExt(Application.ExeName, '.INI' ));
  try
    // Remember Windows positions
    Ini.WriteInteger( 'WindowsPositions', 'PrefTop', FrmPreferences.Top);
    Ini.WriteInteger( 'WindowsPositions', 'PrefLeft', FrmPreferences.Left);
  finally
    Ini.Free;
  end;
end;

end.
